@@ -54,9 +54,9 @@ Version 0.07
 
 =cut
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
-#*****************************************************************
+###################################################################################################
 
 =head1 SYNOPSIS
 
@@ -102,7 +102,7 @@ our $VERSION = '0.07';
 All by default.
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 new - the constructor for a NeurioTools object
 
@@ -137,7 +137,7 @@ sub new {
     }
     
     if (!defined $self->{'neurio'}) {
-      print "NeurioTools->new(): a valid Neurio object is a REQUIRED parameter.\n" if ($self->{'debug'});
+      $self->{'neurio'}->printLOG("NeurioTools->new(): a valid Neurio object is a REQUIRED parameter.\n");
       return 0;
     }
     bless $self, $class;
@@ -145,7 +145,29 @@ sub new {
 }
 
 
-#*****************************************************************
+###################################################################################################
+
+=head2 DESTROY - the destructor for a NeurioTools object
+
+ Destroys a previously created NeuriTools object.
+ 
+=cut
+sub DESTROY {
+    my $self = shift;
+
+    print "\nDestroying ".ref($self)."...\n\n";
+
+    undef $self->{'neurio'};
+    undef $self->{'debug'};
+	undef $self->{'flat_rate'};
+    undef $self->{'TwoTier_rate1'};
+    undef $self->{'TwoTier_cutoff'};
+    undef $self->{'TwoTier_rate2'};
+	undef $self->{'timezone'};
+}
+
+
+###################################################################################################
 
 =head2 set_flat_rate - set the rate charged by your electicity provider
 
@@ -166,16 +188,16 @@ sub set_flat_rate {
 	
     if (defined $rate) {
 	  $self->{'flat_rate'} = $rate;
-      print "NeurioTools->set_flat_rate(): $self->{'flat_rate'}\n" if ($self->{'debug'});
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG("NeurioTools->set_flat_rate(): $self->{'flat_rate'}\n");}
 	  return 1;
     } else {
-      print "NeurioTools->set_flat_rate(): No rate specified\n" if ($self->{'debug'});
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG("NeurioTools->set_flat_rate(): No rate specified\n");}
       return 0;
     }
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 set_TwoTier_rate - set the two tier rates charged by your electicity provider
 
@@ -205,18 +227,18 @@ sub set_TwoTier_rate {
       $self->{'TwoTier_rate1'}  = $rate1;
       $self->{'TwoTier_cutoff'} = $cutoff;
       $self->{'TwoTier_rate2'}  = $rate2;
-      print "NeurioTools->set_TwoTier_rate(): $self->{'TwoTier_rate1'}\n"  if ($self->{'debug'});
-      print "NeurioTools->set_TwoTier_rate(): $self->{'TwoTier_cutoff'}\n" if ($self->{'debug'});
-      print "NeurioTools->set_TwoTier_rate(): $self->{'TwoTier_rate2'}\n"  if ($self->{'debug'});
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG("NeurioTools->set_TwoTier_rate(): $self->{'TwoTier_rate1'}\n");}
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG("NeurioTools->set_TwoTier_rate(): $self->{'TwoTier_cutoff'}\n");}
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG("NeurioTools->set_TwoTier_rate(): $self->{'TwoTier_rate2'}\n");}
       return 1;
     } else {
-      print "NeurioTools->set_TwoTier_rate(): No rate specified\n" if ($self->{'debug'});
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG("NeurioTools->set_TwoTier_rate(): No rate specified\n");}
       return 0;
     }
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_flat_rate - return the flat rate charged by your electicity provider
 
@@ -236,7 +258,7 @@ sub get_flat_rate {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_TwoTier_rate - return the cutoff and two tier rates charged by your electicity provider
 
@@ -256,7 +278,7 @@ sub get_TwoTier_rate {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 set_ISO8601_timezone - set the timezone offset for ISO8601
 
@@ -287,13 +309,13 @@ sub set_ISO8601_timezone {
       substr($tz,3,0,":");
     }
     $self->{'timezone'} = $tz;
-    print "NeurioTools->set_ISO8601_timezone(): $self->{'timezone'}\n" if ($self->{'debug'});
+    if ($self->{'debug'}) {$self->{'neurio'}->printLOG("NeurioTools->set_ISO8601_timezone(): $self->{'timezone'}\n");}
     
     return 1;
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_flat_cost - calculate the cost of consumed power for the specified period
 
@@ -319,7 +341,7 @@ sub get_flat_cost {
     my $i=0;
     
     if ($self->{'flat_rate'} == 0 ) {
-        print "NeurioTools->get_flat_cost(): Cannot calculate cost since rate is set to zero\n" if ($self->{'debug'});
+        if ($self->{'debug'}) {$self->{'neurio'}->printLOG("NeurioTools->get_flat_cost(): Cannot calculate cost since rate is set to zero\n");}
         return 0;
     }
     
@@ -330,7 +352,7 @@ sub get_flat_cost {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_TwoTier_cost - calculate the cost of consumed power for the specified period
 
@@ -357,7 +379,7 @@ sub get_TwoTier_cost {
     my $cost = 0;
     
     if ($self->{'TwoTier_rate1'} == 0 or $self->{'TwoTier_cutoff'} == 0 or $self->{'TwoTier_rate2'} == 0) {
-        print "NeurioTools->get_TwoTier_cost(): Cannot calculate cost since a parameter is set to zero\n" if ($self->{'debug'});
+        if ($self->{'debug'}) {$self->{'neurio'}->printLOG("NeurioTools->get_TwoTier_cost(): Cannot calculate cost since a parameter is set to zero\n");}
         return 0;
     }
     
@@ -375,7 +397,7 @@ sub get_TwoTier_cost {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_kwh_consumed - kwh of consumed power for the specified period
 
@@ -420,7 +442,7 @@ sub get_kwh_consumed {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_kwh_generated - kwh of generated power for the specified period
 
@@ -465,7 +487,7 @@ sub get_kwh_generated {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_energy_consumed - energy consumed for the specified period
 
@@ -508,16 +530,48 @@ sub get_energy_consumed {
 }
 
 
-#*****************************************************************
+###################################################################################################
+
+=head2 get_number_of_appliances - return the number of appliances
+
+ Returns the number of appliances defined in the system.
+
+   $NeurioTools->get_appliance_ID();
+ 
+   This method accepts no parameters:
+ 
+ Returns the number of appliances on success 
+ Returns 0 on failure
+ 
+=cut
+
+sub get_number_of_appliances {
+    my ($self) = @_;
+    my $count  = 0;
+    my $data   = $self->{'neurio'}->fetch_Appliances();
+    
+    if ((defined $data) && ($data !=0)) {
+      foreach my $appliance (@{$data}) {
+        $count++;
+      }
+    } else {
+      $self->{'neurio'}->printLOG("  get_number_of_appliances: Failed to Fetch Appliance ID\n");
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG(Dumper($data));}
+    }
+    return $count;
+}
+
+###################################################################################################
 
 =head2 get_appliance_ID - return the id for the appliance specified
 
- Returns the appliance ID for the name specified.
+ Returns the appliance ID for the name and label specified.
 
-   $NeurioTools->get_appliance_ID($name);
+   $NeurioTools->get_appliance_ID($name,$label);
  
    This method accepts the following parameters:
-     - $name       : textual name of appliance - Required
+     - $name  : textual name of appliance - Required
+     - $label : textual label of appliance - Optional
  
  Returns the appliance ID on success 
  Returns 0 on failure
@@ -526,20 +580,116 @@ sub get_energy_consumed {
 
 sub get_appliance_ID {
     my ($self,$name) = @_;
+    my $data         = $self->{'neurio'}->fetch_Appliances();
     my $appliance;
     
-    my $data = $self->{'neurio'}->fetch_Appliances();
-    if ($data != 0) {
+    if ((defined $data) && ($data !=0)) {
       foreach $appliance (@{$data}) {
         if ($appliance->{'name'} eq $name) {
-           return $appliance->{'id'};
+          $self->{'neurio'}->printLOG("  appliance_ID: ".$appliance->{'id'}."\n");
+          return $appliance->{'id'};
         }
       }
+    } else {
+      $self->{'neurio'}->printLOG("  get_appliance_ID: Failed to Fetch Appliance ID\n");
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG(Dumper($data));}
     }
+    $self->{'neurio'}->printLOG("  get_appliance_ID: No Appliance ID found for $name\n");
     return 0;
 }
 
-#*****************************************************************
+###################################################################################################
+
+=head2 get_cycle_ID - return the id for the most recent cycle
+
+ Returns the ID for the most recent cycle.
+
+   $NeurioTools->get_cycle_ID();
+ 
+   This method accepts no parameters
+ 
+ Returns the cycle ID on success 
+ Returns 0 on failure
+ 
+=cut
+
+sub get_cycle_ID {
+    my $self = shift;
+    my $cycle;
+    my $data = $self->{'neurio'}->fetch_Cycles_by_Sensor();
+
+    if ((defined $data) && ($data !=0)) {
+      $self->{'neurio'}->printLOG("  cycle_ID: ".$data->[0]->{'id'}."\n");
+      return $data->[0]->{'id'};
+    } else {
+      $self->{'neurio'}->printLOG("  get_cycle_ID: Failed to Fetch Cycle ID\n");
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG(Dumper($data));}
+      return 0;
+    }
+}
+
+###################################################################################################
+
+=head2 get_cycle_group_ID - return the id for the most recent cycle group
+
+ Returns the ID for the most recent cycle group.
+
+   $NeurioTools->get_cycle_group_ID();
+ 
+   This method accepts no parameters
+ 
+ Returns the cycle group ID on success 
+ Returns 0 on failure
+ 
+=cut
+
+sub get_cycle_group_ID {
+    my $self = shift;
+    my $data = $self->{'neurio'}->fetch_Cycle_Groups_by_Sensor();
+
+    if ((defined $data)  && ($data !=0)){
+      $self->{'neurio'}->printLOG("  cycle_Group_ID: ".$data->[0]->{'id'}."\n");
+      return $data->[0]->{'id'};
+    } else {
+      $self->{'neurio'}->printLOG("  get_cycle_group_ID: Failed to Fetch Cycle Group ID\n");
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG(Dumper($data));}
+      return 0;
+    }
+    }
+
+###################################################################################################
+
+=head2 get_edges_ID - return an edges id
+
+ Returns and edges ID.
+
+   $NeurioTools->get_edges_ID();
+ 
+   This method accepts no parameters
+ 
+ Returns the edges ID on success 
+ Returns 0 on failure
+ 
+=cut
+
+sub get_edge_ID {
+    my $self = shift;
+    my $data = $self->{'neurio'}->fetch_Cycles_by_Sensor();
+
+    if ((defined $data->[0]->{'upEdge'}) && ($data != 0)) {
+      $self->{'neurio'}->printLOG("  edges_ID: ".$data->[0]->{'upEdge'}->{'id'}."\n");
+      return $data->[0]->{'upEdge'}->{'id'};
+    } elsif ((defined $data->[0]->{'downEdge'}) && ($data != 0)) {
+      $self->{'neurio'}->printLOG("  edges_ID: ".$data->[0]->{'downEdge'}."\n");
+      return $data->[0]->{'downEdge'};
+    } else {
+      $self->{'neurio'}->printLOG("  get_edge_ID: Failed to Fetch Edge\n");
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG(Dumper($data));}
+      return 0;
+    }
+}
+
+###################################################################################################
 
 =head2 get_power_consumed - power consumed for the specified period
 
@@ -581,7 +731,7 @@ sub get_power_consumed {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_ISO8601_time - convert linux time to the time part of ISO8601
 
@@ -602,7 +752,7 @@ sub get_ISO8601_time {
     my $ISO8601_time;
     
     if (!defined $time) {
-      print "\$$time is a required parameter\n" if ($self->{'debug'});
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG("\$$time is a required parameter\n");}
       return 0;
     } else {
       my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime($time);
@@ -612,7 +762,7 @@ sub get_ISO8601_time {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_ISO8601_date - convert linux time to the date part of ISO8601
 
@@ -633,7 +783,7 @@ sub get_ISO8601_date {
     my $ISO8601_date;
     
     if (!defined $time) {
-      print "\$$time is a required parameter\n" if ($self->{'debug'});
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG("\$$time is a required parameter\n");}
       return 0;
     } else {
       my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime($time);
@@ -643,7 +793,7 @@ sub get_ISO8601_date {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_ISO8601_timezone - return the ISOS8601 timezone offset
 
@@ -664,7 +814,7 @@ sub get_ISO8601_timezone {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head2 get_ISO8601 - return the entire ISOS8601 formatted date/time/timezone
 
@@ -686,7 +836,7 @@ sub get_ISO8601 {
     my $ISO8601;
     
     if (!defined $time) {
-      print "\$$time is a required parameter\n" if ($self->{'debug'});
+      if ($self->{'debug'}) {$self->{'neurio'}->printLOG("\$$time is a required parameter\n");}
       return 0;
     } else {
       my $tz = $self->get_ISO8601_timezone();
@@ -697,7 +847,7 @@ sub get_ISO8601 {
 }
 
 
-#*****************************************************************
+###################################################################################################
 
 =head1 AUTHOR
 
